﻿import math
import imp
imp.load_source("util","ScriptEditor/util.py")  
from util import *
imp.load_source("dynamicBoundary","ScriptEditor/dynamicBoundary.py")  
from dynamicBoundary import *
import copy

translator.install()
			
def computeLag(simulator,\
				currentGeometry,\
				currentFill,\
				currentBoundary,\
				mainSimulation,\
				mainSimulationDescriptors,\
				Input,\
				Output,\
				OutputDescriptors,\
				boundaryGroups):

	if "width" in mainSimulation:
		width = mainSimulation["width"]
	else:
		width = 0.0
		
	if "u" in mainSimulation:
		staticU = mainSimulation["u"]
	else:
		staticU = 0.0
				
	computationBoundaryConditions = simulator.getBoundaryConditions(currentGeometry, currentFill, currentBoundary)
	
	# Search for internal group
	internalGroupID = 0
	groupID = 0
	
	simulationSettings = simulator.getSimulationSettings(currentGeometry, currentFill, currentBoundary)
	transientSimulationSettings = copy.deepcopy(simulationSettings["transient"])
	
	invertMainInExCouple = False
	if "lag_invert_main_in_ex" in transientSimulationSettings:
		invertMainInExCouple = transientSimulationSettings["lag_invert_main_in_ex"]
	
	if not invertMainInExCouple:
		mainInternalBoundaryID = mainSimulation["advanced_results"]["boundary"]["main_internal_boundary_id"];
		mainExternalBoundaryID = mainSimulation["advanced_results"]["boundary"]["main_external_boundary_id"];
	else:
		mainExternalBoundaryID = mainSimulation["advanced_results"]["boundary"]["main_internal_boundary_id"];
		mainInternalBoundaryID = mainSimulation["advanced_results"]["boundary"]["main_external_boundary_id"];
		
	internalGroupID = -1
	externalGroupID = -1
	for boundaryGroup in boundaryGroups:
		for boundaryID in boundaryGroup:
			if boundaryID == mainInternalBoundaryID:
				internalGroupID = groupID			
			elif boundaryID == mainExternalBoundaryID:
				externalGroupID = groupID			
		groupID = groupID + 1
		
	print(mainInternalBoundaryID)
	print(mainExternalBoundaryID)
		
	if internalGroupID == -1 or externalGroupID == -1:
		return
	
	internalGroup = boundaryGroups[internalGroupID]
	externalGroup = boundaryGroups[externalGroupID]
	
	zPrefixes = [["z_11_", "z_21_"], ["z_12_", "z_22_"]]
	zNames = [["Z11", "Z21"], ["Z12", "Z22"]]
	
	transientSimulationSettings["graphs_time_unit"] = "hour"
	lagPrecision = transientSimulationSettings["lag_precision"]
	discardSimulation = True
	if "discard_lag_simulation" in transientSimulationSettings:
		discardSimulation = transientSimulationSettings["discard_lag_simulation"]
	
	stimulusPeriod = 86400
	
	if lagPrecision == "low":
		numPeriods = 3
		numLastPeriodSteps = 200
		numSimplePeriodsSteps = 100
	elif lagPrecision == "medium":
		numPeriods = 4
		numLastPeriodSteps = 600
		numSimplePeriodsSteps = 200
	elif lagPrecision == "high":
		numPeriods = 10
		numLastPeriodSteps = 5000
		numSimplePeriodsSteps = 300
	elif lagPrecision == "very_high":
		numPeriods = 12
		numLastPeriodSteps = 15000
		numSimplePeriodsSteps = 400
	
	stepsToSave = numLastPeriodSteps
	stepsToSaveSkip = 1
	maxNumSavedSteps = 200
	while stepsToSave > maxNumSavedSteps:
		stepsToSaveSkip = stepsToSaveSkip + 1
		stepsToSave = numLastPeriodSteps / stepsToSaveSkip
		
	numSteps = list()
	for period in range(numPeriods - 1):
		numSteps.append(numSimplePeriodsSteps)
	numSteps.append(numLastPeriodSteps)
	numSubSteps = 1
	stimulusAmplitude = 20.0
	stimulusOffset = 0.0
	lagResult = dict()

	boundaryVariant = "flows"
	Input["progress_bar_multiplier"] = ((numSimplePeriodsSteps + 1) * (numPeriods - 1) + numLastPeriodSteps + 1) * 2
	Input["boundary_variant"] = boundaryVariant
	
	for subResultID in range(2):
		# Perform static simulation at time 0
		if subResultID == 0:
			# Z11: internal boundary set to adiabatic and external with stimulus
			for boundaryID in internalGroup:
				computationBoundaryConditions[boundaryID][boundaryVariant]["base"] = "none"
				computationBoundaryConditions[boundaryID][boundaryVariant]["type"] = "adiabatic"
			for boundaryID in externalGroup:
				computationBoundaryConditions[boundaryID][boundaryVariant]["base"] = "none"
				computationBoundaryConditions[boundaryID][boundaryVariant]["type"] = "environment_air"
				computationBoundaryConditions[boundaryID][boundaryVariant]["air_temperature_type"] = "fixed"
				computationBoundaryConditions[boundaryID][boundaryVariant]["air_temperature"] = 0.0
				computationBoundaryConditions[boundaryID][boundaryVariant]["temperature_type"] = "fixed"
				computationBoundaryConditions[boundaryID][boundaryVariant]["temperature"] = 0.0
				computationBoundaryConditions[boundaryID][boundaryVariant]["flow_rate_type"] = "fixed"
				computationBoundaryConditions[boundaryID][boundaryVariant]["flow_rate"] = 0.0
		elif subResultID == 1:
			# Z12: internal boundary set to zero temperature environment and external with stimulus
			for boundaryID in internalGroup:
				computationBoundaryConditions[boundaryID][boundaryVariant]["base"] = "none"
				computationBoundaryConditions[boundaryID][boundaryVariant]["type"] = "environment_air"
				computationBoundaryConditions[boundaryID][boundaryVariant]["air_temperature_type"] = "fixed"
				computationBoundaryConditions[boundaryID][boundaryVariant]["air_temperature"] = 0.0
				computationBoundaryConditions[boundaryID][boundaryVariant]["temperature_type"] = "fixed"
				computationBoundaryConditions[boundaryID][boundaryVariant]["temperature"] = 0.0
				computationBoundaryConditions[boundaryID][boundaryVariant]["flow_rate_type"] = "fixed"
				computationBoundaryConditions[boundaryID][boundaryVariant]["flow_rate"] = 0.0				
			for boundaryID in externalGroup:
				computationBoundaryConditions[boundaryID][boundaryVariant]["base"] = "none"
				computationBoundaryConditions[boundaryID][boundaryVariant]["type"] = "environment_air"
				computationBoundaryConditions[boundaryID][boundaryVariant]["air_temperature_type"] = "fixed"
				computationBoundaryConditions[boundaryID][boundaryVariant]["air_temperature"] = 0.0
				computationBoundaryConditions[boundaryID][boundaryVariant]["temperature_type"] = "fixed"
				computationBoundaryConditions[boundaryID][boundaryVariant]["temperature"] = 0.0	
				computationBoundaryConditions[boundaryID][boundaryVariant]["flow_rate_type"] = "fixed"
				computationBoundaryConditions[boundaryID][boundaryVariant]["flow_rate"] = 0.0

		specialElementsConfig = list()
		Input["condensation"] = False
		Input["dynamic"] = False
		Input["static_for_dynamic"] = True		
		
		if not discardSimulation:
			zName = zNames[subResultID][0] + " " + zNames[subResultID][1]
			simulationOutput, simulationOutputDescriptors = addSimulation(simulator,\
																			currentGeometry,\
																			currentFill,\
																			currentBoundary,\
																			specialElementsConfig,\
																			computationBoundaryConditions,\
																			Input,\
																			Output,\
																			OutputDescriptors,\
																			"DL" + " " + zName,\
																			"dl" + "_" + zName)
		else:
			simulationOutput, simulationOutputDescriptors = simulator.simulate(currentGeometry,\
																				currentFill,\
																				currentBoundary,\
																				specialElementsConfig,\
																				computationBoundaryConditions,\
																				Input)
																					
		simulationOutput["dynamic_steps"] = list()
		staticAndDynamicAnalysis = simulationOutput["advanced_results"]["static_and_dynamic_global_analysis"]
		staticAndDynamicAnalysisSteps = 1

		prevExternalT = -1000.0
		maxExternalT = -1000.0
		maxExternalTime = 0.0
		avExternalT = 0.0
		prevExternalEnvT = -1000.0
		maxExternalEnvT = -1000.0
		maxExternalEnvTTime = 0.0
		prevInternalT = -1000.0
		maxInternalT = -1000.0
		minInternalT = 1000.0
		maxInternalTTime = 0.0
		avInternalT = 0.0
		prevInternalQRate = -10000.0
		maxInternalQRate = -100000.0
		maxInternalQRateTime = 0.0
		minInternalQRate = 100000.0
		minInternalQRateTime = 0.0
		avInternalQRate = 0.0
		prevExternalQRate = 100000.0
		minExternalQRate = 100000.0
		minExternalQRateTime = 0.0
		avExternalQRate = 0.0
		
		internalTSamples = list()
		externalEnvTSamples = list()
		internalQRateSamples = list()
		externalQRateSamples = list()
		
		dynamicBoundaryTemperatureAnalysis = list()
		dynamicBoundaryGroupTemperatureAnalysis = list()
		dynamicBoundaryFlowAnalysis = list()
		dynamicBoundaryGroupFlowAnalysis = list()
		dynamicBoundaryCondensationAnalysis = list()
		dynamicBoundaryGroupCondensationAnalysis = list()
		
		for period in range(numPeriods):
			numPeriodSteps = numSteps[period]
			stepLength = stimulusPeriod / numPeriodSteps
			Input["dynamic_time_step"] = stepLength;
			
			for step in range(numPeriodSteps):
				
				specialElementsConfig = list()
				Input["condensation"] = False
				Input["dynamic"] = True
				Input["dynamic_sub_steps"] = numSubSteps
				Input["variant"] = "flows"

				stimulus = stimulusOffset + math.sin(step / (numPeriodSteps - 1) * 2.0 * math.pi) * stimulusAmplitude
				time = step / (numPeriodSteps - 1) * stimulusPeriod + period * stimulusPeriod
				
				for boundaryID in externalGroup:
					computationBoundaryConditions[boundaryID]["flows"]["air_temperature"] = stimulus
					computationBoundaryConditions[boundaryID]["flows"]["temperature"] = stimulus

				if not discardSimulation:
					dynamicOutput = dict()
					dynamicOutputDescriptors = list()
					stepSimulationOutput, stepSimulationOutputDescriptors = addSimulation(simulator,\
																						currentGeometry,\
																						currentFill,\
																						currentBoundary,\
																						specialElementsConfig,\
																						computationBoundaryConditions,\
																						Input,\
																						dynamicOutput,\
																						dynamicOutputDescriptors,\
																						_("Dynamic"))
				else:																		
					stepSimulationOutput, stepSimulationOutputDescriptors = simulator.simulate(currentGeometry,\
																								currentFill,\
																								currentBoundary,\
																								specialElementsConfig,\
																								computationBoundaryConditions,\
																								Input)
																					
				simulation_3d = "3d_simulation" in stepSimulationOutput and stepSimulationOutput["3d_simulation"]
				Input["progress_bar_multiplier"] = 0				
				
				if period == numPeriods - 1:
					boundaryAnalysis = stepSimulationOutput["advanced_results"]["boundary"]["tf_analysis"]
					
					internalT = 0.0
					externalT = 0.0
					internalQRate = 0.0
					internalSize = 0.0
					for internalBoundaryID in internalGroup:
						size = getBoundaryAnalysisSize(boundaryAnalysis[internalBoundaryID])
						internalQRate = internalQRate + boundaryAnalysis[internalBoundaryID]["Q"]
						#internalT = internalT + (boundaryAnalysis[internalBoundaryID]["av_t"] - boundaryAnalysis[internalBoundaryID]["Q"] * 0.13) * size
						internalT = internalT + boundaryAnalysis[internalBoundaryID]["av_t"] * size
						internalSize = internalSize + size
					if internalSize > 0.0:
						internalT = internalT / internalSize
						internalQRate = internalQRate / internalSize
						
					externalEnvT = stimulus
					externalQRate = 0.0
					externalSize = 0.0
					for externalBoundaryID in externalGroup:
						size = getBoundaryAnalysisSize(boundaryAnalysis[externalBoundaryID])
						externalT = externalT + boundaryAnalysis[externalBoundaryID]["av_t"] * size
						externalQRate = externalQRate + boundaryAnalysis[externalBoundaryID]["Q"]
						externalSize = externalSize + size
					if externalSize > 0.0:
						externalT = externalT / internalSize
						externalQRate = externalQRate / externalSize
					
					if step != 0:
						if internalT > maxInternalT and internalT > prevInternalT:
							maxInternalT = internalT
							maxInternalTTime = time
						if internalT < minInternalT and internalT < prevInternalT:
							minInternalT = internalT
						if externalT > maxExternalT and externalT > prevExternalT:
							maxExternalT = externalT
							maxExternalTTime = time
						if externalEnvT > maxExternalEnvT and externalEnvT > prevExternalEnvT:
							maxExternalEnvT = externalEnvT
							maxExternalEnvTTime = time
						if internalQRate > maxInternalQRate and internalQRate > prevInternalQRate:
							maxInternalQRate = internalQRate
							maxInternalQRateTime = time			
						if internalQRate < minInternalQRate and internalQRate < prevInternalQRate:
							minInternalQRate = internalQRate
							minInternalQRateTime = time			
						if externalQRate < minExternalQRate and externalQRate < prevExternalQRate:
							minExternalQRate = externalQRate
							minExternalQRateTime = time
						
					prevInternalT = internalT
					prevExternalT = externalT
					prevExternalEnvT = externalEnvT
					prevInternalQRate = internalQRate
					prevExternalQRate = externalQRate
						
					avInternalT = avInternalT + internalT
					avExternalT = avExternalT + externalT
					avInternalQRate = avInternalQRate + internalQRate
					avExternalQRate = avExternalQRate + externalQRate
						
					internalTSamples.append(internalT)
					externalEnvTSamples.append(externalEnvT)
					internalQRateSamples.append(internalQRate)
					externalQRateSamples.append(externalQRate)
					
					if not discardSimulation and step % stepsToSaveSkip == 0:
						# Store only necessary data in output
						minimalOutput = dict()					
						minimalOutput["simulation_result"] = stepSimulationOutput["simulation_result"]
						minimalOutput["simulation_analysis"] = stepSimulationOutput["simulation_analysis"]
						simulationOutput["dynamic_steps"].append(minimalOutput)					

						relativeTime = time - (stimulusPeriod * (numPeriods - 1))
						addBoundaryDynamicTemperatureAnalysisStep(relativeTime,\
																computationBoundaryConditions,\
																boundaryGroups,\
																boundaryAnalysis,\
																dynamicBoundaryTemperatureAnalysis,\
																dynamicBoundaryGroupTemperatureAnalysis)

						addBoundaryDynamicFlowAnalysisStep(relativeTime,\
															stepLength,\
															computationBoundaryConditions,\
															boundaryGroups,\
															boundaryAnalysis,\
															dynamicBoundaryFlowAnalysis,\
															dynamicBoundaryGroupFlowAnalysis)

						addBoundaryDynamicCondensationAnalysisStep(relativeTime,\
																	computationBoundaryConditions,\
																	boundaryGroups,\
																	boundaryAnalysis,\
																	dynamicBoundaryCondensationAnalysis,\
																	dynamicBoundaryGroupCondensationAnalysis,\
																	simulation_3d)
																	
						combineAnalysis(staticAndDynamicAnalysis, stepSimulationOutput["advanced_results"]["static_and_dynamic_global_analysis"])
						staticAndDynamicAnalysisSteps = staticAndDynamicAnalysisSteps + 1

		avInternalT = avInternalT / numPeriodSteps
		#avInternalT = (minInternalT + maxInternalT) / 2.0
		avInternalQRate = avInternalQRate / numPeriodSteps
		avExternalT = avExternalT / numPeriodSteps
		avExternalQRate = avExternalQRate / numPeriodSteps
		
		zModuli = [0.0, 0.0]
		zArguments = [0.0, 0.0]
		if subResultID == 0:
			if maxInternalT != 0.0:
				# Z_11
				zModuli[0] = maxExternalEnvT / (maxInternalT - avInternalT)
				zArguments[0] = -(maxExternalEnvTTime - maxInternalTTime) / stimulusPeriod * 2.0 * math.pi
				# Z_21
				zModuli[1] = (-minExternalQRate - avInternalQRate) / (maxInternalT - avInternalT)
				zArguments[1] = -(minExternalQRateTime - maxInternalTTime) / stimulusPeriod * 2.0 * math.pi# - math.pi
				#if zArguments[1] < -math.pi:
				#	zArguments[1] = zArguments[1] + 2.0 * math.pi
		elif subResultID == 1:
			if maxInternalQRate != 0.0:
				# Z_12
				zModuli[0] = maxExternalEnvT / (maxInternalQRate - avInternalQRate)
				zArguments[0] = -(maxExternalEnvTTime - maxInternalQRateTime) / stimulusPeriod * 2.0 * math.pi# - math.pi
				if zArguments[0] < -math.pi:
					zArguments[0] = zArguments[0] + 2.0 * math.pi
				# Z_22
				zModuli[1] = (-minExternalQRate - avExternalQRate) / (maxInternalQRate - avInternalQRate)
				zArguments[1] = -(minExternalQRateTime - maxInternalQRateTime) / stimulusPeriod * 2.0 * math.pi
				#if zArguments[1] > math.pi:
				#	zArguments[1] = zArguments[1] - 2.0 * math.pi
		
		zComponentID = 0
		descriptorPosition = 0
		for zPrefix in zPrefixes[subResultID]:
			lagResult[zPrefix + "in_t_max"] = maxInternalT
			lagResult[zPrefix + "in_t_max_time"] = maxInternalTTime - (stimulusPeriod * (numPeriods - 1))
			lagResult[zPrefix + "ex_t_max"] = maxExternalEnvT
			lagResult[zPrefix + "ex_t_max_time"] = maxExternalEnvTTime - (stimulusPeriod * (numPeriods - 1))
			lagResult[zPrefix + "in_qr_max"] = maxInternalQRate
			lagResult[zPrefix + "in_qr_max_time"] = maxInternalQRateTime - (stimulusPeriod * (numPeriods - 1))
			lagResult[zPrefix + "ex_qr_min"] = minExternalQRate
			lagResult[zPrefix + "ex_qr_min_time"] = minExternalQRateTime - (stimulusPeriod * (numPeriods - 1))
			lagResult[zPrefix + "in_t_samples"] = internalTSamples
			lagResult[zPrefix + "ex_t_samples"] = externalEnvTSamples
			lagResult[zPrefix + "in_qr_samples"] = internalQRateSamples
			lagResult[zPrefix + "ex_qr_samples"] = externalQRateSamples
						
			zModulus = zModuli[zComponentID]
			zArgument = zArguments[zComponentID]			
					
			zName = zNames[subResultID][zComponentID]			
			zCos = math.cos(zArgument)
			zSin = math.sin(zArgument)
			zReal = zModulus * zCos
			zImaginary = zModulus * zSin
			yModulus = 1.0 / zModulus
			yReal = - yModulus * zCos
			yImaginary = yModulus * zSin
			yArgument = math.atan2(yImaginary, yReal)
			
			zString = "%.4f" % zReal
			if zImaginary >= 0.0:
				zString = zString + " + "
			else:
				zString = zString + " - "
			zString = zString + "%.4fi" % math.fabs(zImaginary)
			
			yString = "%.4f" % yReal
			if yImaginary >= 0.0:
				yString = yString + " + "
			else:
				yString = yString + " - "
			yString = yString + "%.4fi" % math.fabs(yImaginary)

			lagResult[zPrefix + "real"] = zReal
			lagResult[zPrefix + "imaginary"] = zImaginary
			
			simulationOutputDescriptors.insert(descriptorPosition, createStringDescriptor(zPrefix, zName, zPrefix, 150))
			descriptorPosition = descriptorPosition + 1
			simulationOutput[zPrefix] = zString
			simulationOutputDescriptors.insert(descriptorPosition, createNumberDescriptor(zPrefix + "mod", "|" + zName + "|", "", "%.4f", zPrefix + "mod"))
			descriptorPosition = descriptorPosition + 1
			simulationOutput[zPrefix + "mod"] = zModulus
			simulationOutputDescriptors.insert(descriptorPosition, createNumberDescriptor(zPrefix + "arg", "arg(" + zName + ")", "h", "%.4f", zPrefix + "arg"))
			descriptorPosition = descriptorPosition + 1
			simulationOutput[zPrefix + "arg"] = zArgument / (2.0 * math.pi) * stimulusPeriod / 3600.0
			
			yPrefix = zPrefix.replace('z', 'y')
			yName = zName.replace('Z', 'Y')
			
			simulationOutputDescriptors.insert(descriptorPosition, createStringDescriptor(yPrefix, yName, yPrefix, 150))
			descriptorPosition = descriptorPosition + 1
			simulationOutput[yPrefix] = yString
			simulationOutputDescriptors.insert(descriptorPosition, createNumberDescriptor(yPrefix + "mod", "|" + yName + "|", "", "%.4f", yPrefix + "mod"))
			descriptorPosition = descriptorPosition + 1
			simulationOutput[yPrefix + "mod"] = yModulus
			simulationOutputDescriptors.insert(descriptorPosition, createNumberDescriptor(yPrefix + "arg", "arg(" + yName + ")", "h", "%.4f", yPrefix + "arg"))
			descriptorPosition = descriptorPosition + 1
			simulationOutput[yPrefix + "arg"] = yArgument / (2.0 * math.pi) * stimulusPeriod / 3600.0
			
			lagResult[zPrefix + "mod"] = zModulus
			lagResult[zPrefix + "arg"] = zArgument
			
			zComponentID = zComponentID + 1

		simulationOutputDescriptors.insert(0, createNumberDescriptor("dynamic_period", _("Period"), "h", "%.0f", "period"))
		simulationOutput["dynamic_period"] = stimulusPeriod / 3600.0
		simulationOutput["dynamic_start_time"] = 0.0
		simulationOutput["dynamic_end_time"] = stimulusPeriod
		
		if not discardSimulation:
			# Boundary analysis
			boundaryConditions = simulator.getBoundaryConditions(currentGeometry, currentFill, currentBoundary)
			
			addDynamicBoundaryAnalysis(transientSimulationSettings,\
										boundaryConditions,\
										boundaryGroups,\
										simulationOutput,\
										simulationOutputDescriptors,\
										dynamicBoundaryTemperatureAnalysis,\
										dynamicBoundaryGroupTemperatureAnalysis,\
										"temp",\
										False)	
										
			addDynamicBoundaryAnalysis(transientSimulationSettings,\
										boundaryConditions,\
										boundaryGroups,\
										simulationOutput,\
										simulationOutputDescriptors,\
										dynamicBoundaryFlowAnalysis,\
										dynamicBoundaryGroupFlowAnalysis,\
										"flow",\
										False)	
										
			addDynamicBoundaryAnalysis(transientSimulationSettings,\
										boundaryConditions,\
										boundaryGroups,\
										simulationOutput,\
										simulationOutputDescriptors,\
										dynamicBoundaryCondensationAnalysis,\
										dynamicBoundaryGroupCondensationAnalysis,\
										"cond",\
										False)	
										
			endCombiningAnalysis(staticAndDynamicAnalysis, staticAndDynamicAnalysisSteps)
		
	if lagResult["z_12_mod"] != 0.0:
		dynamicUModulus = 1.0 / lagResult["z_12_mod"]
	else:
		dynamicUModulus = 0.0
		
	#dynamicUPhase = lagResult["z_12_ex_t_max_time"] - lagResult["z_12_in_qr_max_time"]
	reciprocalZ12Real = lagResult["z_12_real"]
	reciprocalZ12Imaginary = -lagResult["z_12_imaginary"]
	#dynamicUPhase = (math.atan2(-reciprocalZ12Imaginary, -reciprocalZ12Real)) / (2.0 * math.pi) * stimulusPeriod
	dynamicUPhase = (math.atan2(lagResult["z_12_imaginary"], -lagResult["z_12_real"])) / (2.0 * math.pi) * stimulusPeriod

	if staticU > 0.0:
		attenuation = dynamicUModulus / staticU
	else:
		attenuation = 0.0
	
	# Create output descriptor and parameter
	lagDescriptor = dict()
	lagDescriptor["type"] = "hc2d_dynamic_lag_result_dlg"
	lagDescriptor["bind"] = "dynamic_lag"
	lagDescriptor["__tag__"] = "d_lag"
	lagDescriptor["name"] = _("Dynamic lag")
	
	lagResult["dynamic_u_modulus"] = dynamicUModulus
	lagResult["dynamic_u_time_shift"] = dynamicUPhase
	lagResult["u_attenuation"] = attenuation
	lagResult["period_length"] = stimulusPeriod
	
	if lagResult["z_12_mod"] != 0.0:
		lagResult["y_11_mod"] = lagResult["z_11_mod"] / lagResult["z_12_mod"]
		lagResult["y_11_arg"] = lagResult["z_11_arg"] - lagResult["z_12_arg"] - math.pi
		if lagResult["y_11_arg"] < 0.0:
			lagResult["y_11_arg"] = lagResult["y_11_arg"] + 2.0 * math.pi

		lagResult["y_22_mod"] = lagResult["z_22_mod"] / lagResult["z_12_mod"]
		lagResult["y_22_arg"] = lagResult["z_22_arg"] - lagResult["z_12_arg"] - math.pi
		if lagResult["y_22_arg"] < 0.0:
			lagResult["y_22_arg"] = lagResult["y_22_arg"] + 2.0 * math.pi

		tmp1 = (lagResult["z_11_real"] - 1.0) * (lagResult["z_11_real"] - 1.0)
		tmp2 = lagResult["z_11_imaginary"] * lagResult["z_11_imaginary"]
		lagResult["k_1"] = 3600.0 * 24.0 / (2.0 * math.pi) * math.sqrt(tmp1 + tmp2) / lagResult["z_12_mod"]
		
		tmp1 = (lagResult["z_22_real"] - 1.0) * (lagResult["z_22_real"] - 1.0)
		tmp2 = lagResult["z_22_imaginary"] * lagResult["z_22_imaginary"]
		lagResult["k_2"] = 3600.0 * 24.0 / (2.0 * math.pi) * math.sqrt(tmp1 + tmp2) / lagResult["z_12_mod"]
	else:
		lagResult["y_11_mod"] = 0.0
		lagResult["y_11_arg"] = 0.0
		lagResult["y_22_mod"] = 0.0
		lagResult["y_22_arg"] = 0.0
		lagResult["k_1"] = 0.0
		lagResult["k_2"] = 0.0
		
	lagResult["internal_color"] = computationBoundaryConditions[mainInternalBoundaryID]["color"]
	lagResult["external_color"] = computationBoundaryConditions[mainExternalBoundaryID]["color"]
	
	mainSimulation["dynamic_lag"] = lagResult
	mainSimulationDescriptors.append(lagDescriptor)
	
